def is_prime(a, minor_primes): # For decimals
  for p in minor_primes:
    #print(f'{n} y {p}')
    if a % p == 0:
      return False
  return True

# Example:
# is_prime(7, [2, 3, 5]) # True
# is_prime(15, [2, 3, 5, 7, 11, 13])  # False


def generate_primes(until):
    primes = []
    for i in range(2, until):
        if is_prime(i, primes) == True:
            primes.append(i)
    return primes

def decompose(a):
    l = generate_primes(a)
    elements_factor_exponent = {}
    for c in l:
        while a % c == 0:
            if c in elements_factor_exponent:
                elements_factor_exponent[c] += 1
            else:
                elements_factor_exponent[c] = 1
        a = a//c
    return elements_factor_exponent

# decompose(8*9*5*11)  # {2: 3, 3: 2, 5: 1, 11: 1}

def print_descom(d, mult='·'):
    sir = [f"{k}^{v}" for k, v in d.items()]
    return f" {mult} ".join(sir)

# print_descom(descompose(8*9*5*11))
# print_descom(decompose(8*9*5*11), mult='*') # Change symbol

def mcm(a, b):
    a, b = abs(a), abs(b)
    mcd_value = mcd(a, b)
    return abs(a * b) // mcd_value

def mcd(a, b):
    a, b = abs(a), abs(b)
    while b != 0:
        a, b = b, a % b
    return a



class Roman:
    def __init__(self, number: str):
        self.number = number
    def __repr__(self):
        return f"Number: {self.number}"
    def RomanToDec(self):
        numbers = {
            'I':1,
            'V':5,
            'X':10,
            'L':50,
            'C':100,
            'D':500,
            'M':1000,
        }
        total = 0
        itemb = 0
        n = reversed(self.number.upper())
        for i in n:
            value = numbers[i]
            if value < itemb:
                total -= value
            else:
                total += value
                itemb = value
        return total

def DecToRoman(number):
    values = [
        (1000, 'M'), (900, 'CM'), (500, 'D'), (400, 'CD'),
        (100, 'C'), (90, 'XC'), (50, 'L'), (40, 'XL'),
        (10, 'X'), (9, 'IX'), (5, 'V'), (4, 'IV'), (1, 'I')
    ]
    result = ""
    for value, symbol in values:
        while number >= value:
            result += symbol
            number -= value
    return result



import matplotlib.pyplot as plt
from turtle import *
from colors import Color
import time as tm
class Coordinates:
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.coordinates = [x, y]

    def __repr__(self):
        return f"x: {self.x} \ny: {self.y}"
    
    def __add__(self, a, list: bool=False, dict: bool=False, set: bool=False, tuple:bool=False, coor:bool=True):
        if coor:
            ecoor = Coordinates(self.x + a.x, self.y + a.y)
            return ecoor
        if list:
            elist = [self.x + a.x, self.y + a.y]
            return elist
        if dict:
            edict = {
                'x': self.x + a.x,
                'y': self.y + a.y
            }
            return edict
        if set:
            eset = {self.x + a.x, self.y + a.y}
            return eset

        if tuple:
            etuple = (self.x + a.x, self.y + a.y)
            return etuple
        
        else:
            return "Parameters Missing"
    def __sub__(self, a, list: bool=False, dict: bool=True, set: bool=False, tuple:bool=False):
        if (list):
            elist = [self.x - a.x, self.y - a.y]
            return elist
        if (dict):
            edict = {
                'x': self.x - a.x,
                'y': self.y - a.y
            }
            return edict
        if (set):
            eset = {self.x - a.x, self.y - a.y}
            return eset

        if (tuple):
            etuple = (self.x - a.x, self.y - a.y)
            return etuple
        
        else:
            return "Parameters Missing"
        

class Segments:
    def __init__(self, x: list):
        self.x = x
    
    def draw(self, color: Color="#000000"):
        for x, y in self.x:
            plt.scatter(x, y, c=color.hex)
        plt.show()

class Rectangles:
    def __init__(self, alture, base):
        self.h = alture
        self.b = base

    def draw(self):
        home()
        forward(self.h)
        left(self.b)
        tm.sleep(40)


"""
def SegToCoor(a: Segments):
    s = 0
    s2 = 0
    result = cp.Coordinates(s, s2)
    for x, y in a.x:
        s = x
        s2 = y
    return result
# Still in development
"""


from number import *
from geometry import *
from primes import *
from roman import *
from colors import *
# Known numbers:
print(f"pi number: {PI}")
print(f"euler number: {EULER}")
print(f"golden ratio number: {GOLDEN_RATIO}")
print(f"kaprekar number: {KAPREKAR}")

# Coordinates
coordenate = Coordinates(10, -10)
p = Coordinates(-10, 10)
print(coordenate - p) # Returns another coordinate with the subtraction / adding (coordenate + p)

# Color
color = Color("#911B1B")
randomcolor = randomColor() # Creates a random color
print(randomcolor)
# Roman:
roman_Number = Roman("XVII")
print(roman_Number.RomanToDec()) # 17
any_number = 10
print(DecToRoman(any_number)) # X

# Primes:
prime = 17
print(is_prime(prime, [2, 3, 5])) # True (means it's prime)
# The [2, 3, 5] list is another thing which you need to put in there (those are numbers minor than the prime you're checking)
print(generate_primes(40)) # Will generate prime numbers until the parameter you put (must be int) [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37]
# print(decompose(120)) # Will do a factorial decompose with the number you passed (may take a while)
print(mcm(12, 24)) # Returns the answer to the mcm
print(mcd(12, 24)) # Same as mcm but different operation


# Segments (matplotlib)
segment = Segments([(10, -10), (50, -10), (0, 10)])
#segment.draw(color=color)

rectangle = Rectangles(12, 24)
rectangle.draw()


import geometry as gm
from random import randint, choice

class Color:
    def __init__(self, hex: str):
        self.hex = hex

def randomColor():
    return "#" +''.join([choice('0123456789ABCDEF') for j in range(6)])